'use strict';

var _Object$keys = require('babel-runtime/core-js/object/keys')['default'];

var _interopRequireDefault = require('babel-runtime/helpers/interop-require-default')['default'];

var _graphql = require('graphql');

var _lodash = require('lodash');

var _lodash2 = _interopRequireDefault(_lodash);

var _simplifyAST = require('./simplifyAST');

var _simplifyAST2 = _interopRequireDefault(_simplifyAST);

module.exports = function (target, options) {
  var resolver,
      targetAttributes,
      argsToFindOptions,
      generateIncludes,
      isModel = !!target.getTableName,
      isAssociation = !!target.associationType,
      association = isAssociation && target,
      model = isAssociation && target.target || isModel && target;

  targetAttributes = _Object$keys(model.rawAttributes);

  options = options || {};
  if (options.include === undefined) options.include = true;
  if (options.before === undefined) options.before = function (options) {
    return options;
  };
  if (options.after === undefined) options.after = function (result) {
    return result;
  };

  argsToFindOptions = function (args) {
    var result = {};

    if (args) {
      _Object$keys(args).forEach(function (key) {
        if (~targetAttributes.indexOf(key)) {
          result.where = result.where || {};
          result.where[key] = args[key];
        }

        if (key === 'limit' && args[key]) {
          result.limit = args[key];
        }

        if (key === 'order' && args[key]) {
          result.order = [[args[key]]];
        }
      });
    }

    result.logging = options.logging;
    return result;
  };

  generateIncludes = function (simpleAST, type, root) {
    var result = { include: [], attributes: [] };

    type = type.ofType || type;

    _Object$keys(simpleAST).forEach(function (key) {
      var association,
          includeOptions,
          args = simpleAST[key].args,
          includeResolver = type._fields[key].resolve,
          nestedResult,
          allowedAttributes;

      if (!includeResolver) return;

      if (includeResolver.$proxy) {
        while (includeResolver.$proxy) {
          includeResolver = includeResolver.$proxy;
        }
      }

      if (includeResolver.$passthrough) {
        var dummyResult = generateIncludes(simpleAST[key].fields, type._fields[key].type, root);
        result.include = result.include.concat(dummyResult.include);
        return;
      }

      association = includeResolver.$association;

      if (association) {
        includeOptions = argsToFindOptions(args);
        allowedAttributes = _Object$keys(association.target.rawAttributes);

        if (includeResolver.$before) {
          includeOptions = includeResolver.$before(includeOptions, args, root);
        }

        if (options.include && !includeOptions.limit) {
          if (includeOptions.order) {
            includeOptions.order.map(function (order) {
              order.unshift({
                model: association.target,
                as: association.options.as
              });

              return order;
            });

            result.order = (result.order || []).concat(includeOptions.order);
            delete includeOptions.order;
          }

          includeOptions.attributes = _Object$keys(simpleAST[key].fields).filter(function (attribute) {
            return ~allowedAttributes.indexOf(attribute);
          });

          includeOptions.attributes.push(association.target.primaryKeyAttribute);

          nestedResult = generateIncludes(simpleAST[key].fields, type._fields[key].type, root);

          includeOptions.include = (includeOptions.include || []).concat(nestedResult.include);
          includeOptions.attributes = _lodash2['default'].unique(includeOptions.attributes.concat(nestedResult.attributes));

          result.include.push(_lodash2['default'].assign({ association: association }, includeOptions));
        } else if (association.associationType === 'BelongsTo') {
          result.attributes.push(association.foreignKey);
        } else {
          result.attributes.push(model.primaryKeyAttribute);
        }
      }
    });

    return result;
  };

  resolver = function (source, args, root, ast, type) {
    if (association && source.get(association.as)) {
      return source.get(association.as);
    }

    var list = type instanceof _graphql.GraphQLList,
        includeResult,
        simpleAST,
        findOptions = argsToFindOptions(args);

    simpleAST = (0, _simplifyAST2['default'])(ast);
    root = root || {};
    type = type.ofType || type;

    findOptions.attributes = _Object$keys(simpleAST).filter(function (attribute) {
      return ~targetAttributes.indexOf(attribute);
    });

    findOptions.attributes.push(model.primaryKeyAttribute);

    includeResult = generateIncludes(simpleAST, type, root);

    findOptions.include = includeResult.include;
    findOptions.root = root;
    findOptions.attributes = _lodash2['default'].unique(findOptions.attributes.concat(includeResult.attributes));
    findOptions.logging = findOptions.logging || root.logging;

    if (includeResult.order) {
      findOptions.order = (findOptions.order || []).concat(includeResult.order);
    }

    findOptions = options.before(findOptions, args, root);

    if (association) {
      return source[association.accessors.get](findOptions).then(function (result) {
        return options.after(result, args, root, simpleAST);
      });
    }
    return model[list ? 'findAll' : 'findOne'](findOptions).then(function (result) {
      return options.after(result, args, root, simpleAST);
    });
  };

  if (association) {
    resolver.$association = association;
  }

  resolver.$before = options.before;

  return resolver;
};